

/* m3.c -- Read and show values for the max31855. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <signal.h>
#include "max31855.h"

int G_rflag;

/* React to sigints, stop the loop in these cases. */ 
void sigintcatcher(int sig, siginfo_t *sinfo, void *vpx)
{
	fprintf(stderr, "\n%s(%d): Caught SIGINT, will exit...\n", 
		__FILE__, __LINE__); 
	G_rflag = 1;
	return;
}

int main(int argc, char **argv)
{
	char info[256];
	int rdok;
	int j, vb;
	struct sigaction sact; 

	double ta, tr;
	uint8_t fault, ftype;

	G_rflag = 0; 
	memset(&sact, 0, sizeof(struct sigaction) ); 
	sact.sa_sigaction = sigintcatcher; 
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIGINT, &sact, 0); 

	vb = 0;
	for(j = 1; j<argc; j++)
	{
		if(!strcmp("-v", argv[j])) vb = 1; 
	}

	for(;;)
	{
		if(G_rflag) break;

		rdok = readmax31855("/dev/spidev0.0", info, 256, 
			&ta, &tr, &fault, &ftype); 

		if(rdok < 0)
		{
			fprintf(stderr, "%s(%d): %s\n", __FILE__, __LINE__, info);
			continue;
		}

		if(fault || ftype)
		{
			fprintf(stderr, "%s(%d): Fault %d type %d\n", 
				__FILE__, __LINE__, fault, ftype);
			continue;
		}	

		if(vb)
		{
			fprintf(stdout, "%s(%d): T=%G C Tref=%G C Fault %d type %d\n", 
					__FILE__, __LINE__, ta, tr, fault, ftype);
		}
		else
		{
			fprintf(stdout, "\rT=%.2f C", ta); 
			fflush(stdout);
		}
	}

	fprintf(stderr, "\n%s(%d): Stop ...\n", __FILE__, __LINE__); 
	return(0);
}






