

/* m3serv.c -- Read and show values for the max31855. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <utilib.h>

#include "max31855.h"
#include "tcpserver.h"

int G_rflag;

/* React to sigints, stop the loop in these cases. */ 
void sigintcatcher(int sig, siginfo_t *sinfo, void *vpx)
{
	fprintf(stderr, "\n%s(%d): Caught SIGINT, will exit...\n", 
		__FILE__, __LINE__); 
	G_rflag = 1;
	return;
}

/* Tell about failures and status info */
int fdetect(int rdok, int fault, int ftype, char *txbuff, size_t tbufmax)
{
	if(rdok < 0)
	{
		snprintf(txbuff, tbufmax, "Failed to access device"); 
		return(1);
	}

	if(fault || ftype)
	{
		snprintf(txbuff, 512, "Fault %d type %d", fault, ftype); 
		return(1);
	}
	return(0);
}


/* on accept, look at the request, make the measurement 
	Return 0 to continue, != 0 to stop.
*/ 
int m3function(int csd, void *environ)
{
	char info[256];
	int rdok;
	int fdet;

	double ta, tr;
	uint8_t fault, ftype;

	int rval;
	char rbuff[512];
	char comd[20];
	char *stop; 
	char txbuff[512]; 

	memset(rbuff, 0, 512); 
	recv(csd, rbuff, 512, 0);

	rval = 0;
	stop = rbuff;
	for(;;)
	{
		strword(stop, &stop, comd, 20); 
		if(notext(comd)) break; 

		if(!strcmp("quit", comd))
		{
			rval = 1;
			snprintf(txbuff, 512, "Quit received, will stop");
			send(csd, txbuff, strlen(txbuff), 0); 
		}
		else
        if(!strcmp("?", comd))
        {
            strcpy(txbuff, "Commands:\n");
            send(csd, txbuff, strlen(txbuff), 0);
            strcpy(txbuff, "         T    - Thermocouple temperature\n");
            send(csd, txbuff, strlen(txbuff), 0);
            strcpy(txbuff, "         R    - Reference temperature\n");
            send(csd, txbuff, strlen(txbuff), 0);
            strcpy(txbuff, "         S    - Status info\n");
            send(csd, txbuff, strlen(txbuff), 0);
            strcpy(txbuff, "         quit - stop the server program\n");
            send(csd, txbuff, strlen(txbuff), 0);
            strcpy(txbuff, "         ?    - This list\n");
            send(csd, txbuff, strlen(txbuff), 0);
        }
		else
        if(!strcmp("T", comd))
		{
			rdok = readmax31855("/dev/spidev0.0", info, 256, 
				&ta, &tr, &fault, &ftype); 

			fdet = fdetandinfo31855(rdok, fault, ftype, txbuff, 512);

			if(!fdet)
			{
				snprintf(txbuff, 512, "T=%.2f C", ta); 
			}
			send(csd, txbuff, strlen(txbuff), 0);
		}
		else
        if(!strcmp("R", comd))
		{
			rdok = readmax31855("/dev/spidev0.0", info, 256, 
				&ta, &tr, &fault, &ftype); 

			fdet = fdetandinfo31855(rdok, fault, ftype, txbuff, 512);

			if(!fdet)
			{
				snprintf(txbuff, 512, "T=%.2f C", tr); 
			}
			send(csd, txbuff, strlen(txbuff), 0);
		}
		else
        if(!strcmp("S", comd))
        {
			rdok = readmax31855("/dev/spidev0.0", info, 256, 
				&ta, &tr, &fault, &ftype); 

			if(rdok < 0)
			{
				fdetandinfo31855(rdok, fault, ftype, txbuff, 512);
			}
			else
			{
				snprintf(txbuff, 512, 
					"T=%G C Tref=%G C Fault %d type %d\n", ta, tr, fault, ftype);
			}
			send(csd, txbuff, strlen(txbuff), 0);
		}
	}
	return(rval);
}



int main(int argc, char **argv)
{
	struct sigaction sact; 

    unsigned short portno;
    int verbose;
    int j;
    char *stop;

	G_rflag = 0; 

	memset(&sact, 0, sizeof(struct sigaction) ); 
	sact.sa_sigaction = sigintcatcher; 
	sact.sa_flags = SA_SIGINFO | SA_RESTART; 
	sigaction(SIGINT, &sact, 0); 

    /* Default port number. IP-address is/should be the one on the local machine. */
    portno = 9998;
    verbose = 0;

    /* Work through the args.
        -p portno
        -v
    */
    for(j = 1; j<argc; j++)
    {
        if(!strcmp("-v", argv[j])) verbose = 1;
        else
        if(!strcmp("-p", argv[j]))
        {
            if(j+1 < argc)
                portno = (unsigned short) strtol(argv[j+1], &stop, 10);
        }
    }

    if(portno == 0)
    {
        fprintf(stderr, "Usage: %s -p port -v\n", argv[0]);
        return(1);
    }

    TCPserverSingle(portno, verbose, m3function, NULL);
    return(0);
}






