

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdint.h>


/* For SPI */
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>

#include "spiio.h"


int main()
{
	char spidevice[90];
	char errbuf[128];
	int fd, oks;
	int nrd;

	double ta, tr;
	uint32_t val32, fault, ftype;
	int16_t tcv25, trv25;

	strcpy(spidevice, "/dev/spidev0.0"); 

	fd = open(spidevice, O_RDWR);
	if(fd < 0)
	{
		strerror_r(errno, errbuf, 128);
		fprintf(stderr, "%s: %s\n", spidevice, errbuf); 
		return(-1);	
	}

	oks = setupspidev(fd, SPI_MODE_1, errbuf, 128); 
	if(oks < 0)
	{
		fprintf(stderr, "Setup spidev %s\n", errbuf); 
		close(fd);
		return(-1);	
	}

	nrd = read32(fd, &val32); 
	if(nrd < 0)
	{
		fprintf(stderr, "read error %s\n", errbuf); 
		close(fd);
		return(-1);	
	}


	fprintf(stderr, "%s(%d): Read 0x%08x\n", __FILE__, __LINE__, val32); 

	/* Bits: 31-18 2-complement signed temperature value (+1600 to -250)
			 17
             16    1=Fault 0=ok
			 15-4  2-complement signed reference junction temperature (-55 to +127)
			 3
             2     1=Fault: TC shorted to vcc
             1     1=Fault: TC shorted to gnd
             0     1=Fault: TC is open
	*/ 

	tcv25 = (val32 >> 18) & 0x0CFFF;
	trv25 = (val32 >> 4) & 0x0FFF;
	tr = (double) (trv25) / 16.0; 
	fault = (val32 & 0x10000);
	ftype = (val32 & 0x07);

	/* Sign-extend the tcv25. */
	if(tcv25 & 0x0800) 
	{
		tcv25 |= 0xF000; 
	}

	ta = (double) (tcv25) / 4.0; 

	fprintf(stderr, "%s(%d): T=0x%04x %G C Tref=0x%03x %G C Fault %d type %d\n", 
		__FILE__, __LINE__, tcv25, ta, trv25, tr, fault, ftype);


	close(fd);
	return(0);
}




