

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdint.h>


/* For SPI */
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>

int main()
{
	char spidevice[90];
	char errbuf[128];
	int fd, oks, bits, mode, speed;
	char rdx[6];
	int nrd;
	uint16_t rh, rl, rx; 
	double ta;
	int disconnd;

	strcpy(spidevice, "/dev/spidev0.1"); 

	fd = open(spidevice, O_RDWR);
	if(fd < 0)
	{
		strerror_r(errno, errbuf, 128);
		fprintf(stderr, "%s: %s\n", spidevice, errbuf); 
		return(-1);	
	}
	
	mode = SPI_MODE_0;
	oks = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		fprintf(stderr, "Setting device-mode to %d: %s\n", mode, errbuf); 
		close(fd);
		return(-1);	
	}

	bits = 8;
	oks = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		fprintf(stderr, "Setting %d bits: %s\n", bits, errbuf); 
		close(fd);
		return(-1);	
	}

	speed = 100000;
	oks = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		fprintf(stderr, "Setting speed %d Hz: %s\n", speed, errbuf); 
		close(fd);
		return(-1);	
	}


	nrd = read(fd, rdx, 2);

	// printf("Read %d (0x%x.%x.%x.%x)\n", nrd, rdx[0], rdx[1], rdx[2], rdx[3]); 

	/* 0ttt tttt tttt t i03 */

	rh = rdx[0];
	rl = rdx[1];

	rx = (rh << 8) + rl;
	ta = (rx & 0xFFF8) / 32.0;

	disconnd = rx & 4;

	if(disconnd == 0)
	{
		// printf("Read %d (0x%x.%x) 0x%x %G\n", nrd, rh, rl, rx, ta); 
		printf("%G C\n", ta); 
	}
	else
	{
		// printf("Read %d (0x%x.%x) 0x%x probe is not grounded\n", nrd, rh, rl, rx); 
		printf("Probe is not grounded\n"); 

	}

	close(fd);
	return(0);
}




