
#ifndef TCPSERVERHINCLUDED 
#define TCPSERVERHINCLUDED 1

/* Run the server loop. 
	This may return if the inside function decided to break, 
	in which case it returns a value from that, or 
	there might be some other error, in which case -1 is returned
	and errno is set. 

	The workfunction is called as:

	(*workfunction)(clientsockethandle, exparam); 

	and should perform all the required or desired recv() and send()
	actions itself.  It should return 0 to keep the server going, 
	and any other value to stop the server. 
*/ 

int TCPserverSingle(unsigned short portno, int verbose, 
	int (*workfunction)(), void *exparam);


/* Run multiprocess loop. 
	This may return if the inside function decided to break, 
	in which case it returns a value from that, or 
	there might be some other error, in which case -1 is returned
	and errno is set. 

	The workfunction is called as:

	(*workfunction)(clientsockethandle, exparam); 

	and should perform all the required or desired recv and send
	actions itself.  It should return 0 to keep the server going, 
	and any other value to stop the server. 

	if runuid is nonzero, the server is expected to start run as root 
	and then change the effective uid to runuid.
*/ 

int TCPserverMulti(unsigned short portno, int verbose, 
	int (*workfunction)(), void *exparam, uid_t runuid);

#endif

