
#include <stdio.h>

#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "tcpserver.h"

/* Eat SIGHUPs */ 
void sighupcatcher(int sig, siginfo_t *sinfo, void *vpx)
{
	return;
}

/* Run the server loop. 
	This may return if the inside function decided to break, 
	in which case it returns a value from that, or 
	there might be some other error, in which case -1 is returned
	and errno is set. 

	The workfunction is called as:

	(*workfunction)(clientsockethandle, exparam); 

	and should perform all the required or desired recv and send
	actions itself.  It should return 0 to keep the server going, 
	and any other value to stop the server. 
*/ 

int TCPserverSingle(unsigned short portno, int verbose, 
	int (*workfunction)(), void *exparam) 
{
	struct sigaction sact; 

	int hs;
	struct sockaddr_in addr;
	int cc;

	unsigned int alen;
	struct sockaddr_in cliaddr;
	int csd; 
	int wz;

	/* Eat SIGHUPs */ 
	bzero(&sact, sizeof(struct sigaction) ); 
	sact.sa_sigaction = sighupcatcher;
	sact.sa_flags = SA_SIGINFO | SA_RESTART;
	sigaction(SIGHUP, &sact, 0);

#if(0)
	/* Unless we are being chatty about what is going on 
		inside our program, Go into "Tommy"-mode (as in Tommy 
		the deaf, dumb, and blind kid from the Who's rock opera). 
		This appears to be what is needed for the 
		process to survive the closing of the 
		outside shell. Eating SIGHUPs don't seem to be enough... */ 
	if(!verbose)
	{
		fclose(stdout);
		fclose(stdin);
		fclose(stderr);
	}
#endif

	errno = 0;
	hs = socket(PF_INET, SOCK_STREAM, 0);

	if(verbose)
		fprintf(stderr, "%s(%d): socket() = %d errno = %d (%s)\n", 
			__FILE__, __LINE__, hs, errno, strerror(errno));

	if(hs < 0) return(-1);

	memset( (char *) &addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(portno);
	addr.sin_addr.s_addr = INADDR_ANY;

	errno = 0;
	cc = bind(hs, (struct sockaddr *) &addr, sizeof(struct sockaddr_in));

	if(verbose)
		fprintf(stderr, "%s(%d): bind(0x%lx,  %d) = %d errno = %d (%s)\n", 
			__FILE__, __LINE__, 
		(unsigned long) addr.sin_addr.s_addr, 
		ntohs(addr.sin_port), 
		cc, errno, strerror(errno));

	if(cc == -1) return(-1);

	errno = 0;
	cc = listen(hs, 20);

	if(verbose)
		fprintf(stderr, "%s(%d): listen() = %d errno = %d (%s)\n", 
			__FILE__, __LINE__, cc, errno, strerror(errno));

	if(cc == -1) return(-1);

	/* This is the server's accept loop where it starts talking with the 
		client. We have just a simple loop for a simple job here, which runs 
		forever.

		be set up to run in the background as a daemon, and it may want to
		receive and act on signals such as SIGHUP, SIGINT, and if it produces
		child processes
	*/ 

	wz = 0; 

	for(;;)
	{
		alen = sizeof(struct sockaddr_in);		
		errno = 0;
		csd = accept(hs, (struct sockaddr *) &cliaddr, &alen);

		if(verbose)
			fprintf(stderr, "%s(%d): Server has accepted: %d\n", 
				__FILE__, __LINE__, csd);

		if(csd > 0)
		{
			/* Call the function that will want to do the recv and send
				work itself. It may want to do this in several goes. */ 
			wz = (*workfunction)(csd, exparam); 

			/* We're done, so just close the client connection. */ 
			shutdown(csd, SHUT_RDWR);
			close(csd);
		}
		else
		{
			if(verbose) 
				fprintf(stderr, "%s(%d): accept() = %d errno = %d (%s)\n", 
					__FILE__, __LINE__, csd, errno, strerror(errno));
		}

		if(wz != 0) break; 
	}
	shutdown(hs, SHUT_RDWR);
	close(hs);
	return(wz);
}



